/*
 * Copyright(C) OASIS(R) 2009,2010. All Rights Reserved.
 * OASIS trademark, IPR and other policies apply.    
 */
package client;

import static org.junit.Assert.assertEquals;

import java.io.IOException;
import java.io.InputStream;
import java.net.URL;
import java.util.Properties;

import javax.xml.namespace.QName;
import javax.xml.ws.Service;

import org.junit.After;
import org.junit.Before;
import org.junit.Test;

import testClient.TestException;
import testClient.TestException_Exception;
import testClient.TestInvocation;

/**
 * A generic test client based on JAX-WS APIs
 */
public class BaseJAXWSTestRunner {

	private static RuntimeBridge runtimeBridge = null;
	// Environment variable names
	private static String RUNTIME_BRIDGE_IMPL_PROPKEY 		= "OASIS_TESTENV_RUNTIME_BRIDGE_CLASS";
	private static String RUNTIME_IMPL_LANG_PROPKEY 		= "OASIS_TESTENV_IMPL_LANG";
	private static String CONTRIBUTION_LOCATION_PROPKEY		= "OASIS_TESTENV_CONTRIBUTION_LOCATION";
	// Property value names
	private static String RUNTIME_BRIDGE_IMPL_PROPNAME 		= "org.oasis.sca.tests.assembly.runtime_bridge";
	private static String RUNTIME_IMPL_LANG_PROPNAME 		= "org.oasis.sca.tests.assembly.lang";
	private static String CONTRIBUTION_LOCATION 			= "org.oasis.sca.tests.assembly.contribution.location";
	private static String DEFAULT_IMPL_LANG = "Java";
	protected static String _Lang = "_Java";
	protected static Properties configProperties;
    protected TestConfiguration testConfiguration;
    static boolean proceed = true;
	
	public BaseJAXWSTestRunner() {
		configProperties = readProperties();
		setupImplLanguage();
		testConfiguration = getTestConfiguration();
		try {
			setupRuntimeBridge();
		} catch (Exception e) {
			System.err.println("Failed to create a bridge class to the runtime under test./n " +
					           "Please ensure the environment variable OASIS_TESTENV_RUNTIME_BRIDGE_CLASS /n" +
					           "or Java property named org.oasis.sca.tests.assembly.runtime_bridge /n" +
					           "is set to a valid implementation of the client.RuntimeBridge interface.");
			e.printStackTrace();
		} // end try
	} // end constructor BaseJAXWSTestCase()
	
	private Properties readProperties() {
		String implLanguage 	= System.getenv(RUNTIME_IMPL_LANG_PROPKEY);
		String bridgeClass 		= System.getenv(RUNTIME_BRIDGE_IMPL_PROPKEY); 
		String locationPattern 	= System.getenv(CONTRIBUTION_LOCATION_PROPKEY);
		Properties properties = new Properties();
		try {
			//properties.load( this.getClass().getResourceAsStream("oasis-sca-tests.properties") );
		    // find the properties file that configures the test harness. First look on the
		    // classpath for a file called "oasis-sca-tests.properties".
		    InputStream propertiesStream = this.getClass().getResourceAsStream("/oasis-sca-tests.properties");
		    
		    // If the properties file can't be found then read the default properties file
		    // called "client.oasis-sca-tests.properties"
		    if (propertiesStream == null){
		        propertiesStream = this.getClass().getResourceAsStream("oasis-sca-tests.properties");
		    } // end if
		    properties.load(propertiesStream);
		} catch (IOException e) {	
			System.out.println("Unable to read oasis-sca-tests.properties file");
		} // end try
		
		if( implLanguage != null ) properties.setProperty(RUNTIME_IMPL_LANG_PROPNAME, implLanguage);
		if( bridgeClass != null ) properties.setProperty(RUNTIME_BRIDGE_IMPL_PROPNAME, bridgeClass);
		if( locationPattern != null ) properties.setProperty(CONTRIBUTION_LOCATION, locationPattern);
		
		if( properties.getProperty(RUNTIME_IMPL_LANG_PROPNAME) == null ) {
			System.err.println("No implementation language specified - using " + DEFAULT_IMPL_LANG + "/n" +
            "Set the OASIS_TESTENV_IMPL_LANG environment variable or org.oasis.sca.tests.assembly.lang property.");
			properties.setProperty(RUNTIME_IMPL_LANG_PROPNAME, DEFAULT_IMPL_LANG );
		} // end if 
		
		return properties;
	} // end method readProperties
	
	private void setupImplLanguage() {
		_Lang = "_" + configProperties.getProperty(RUNTIME_IMPL_LANG_PROPNAME);
		System.out.println("Implementation language set to: " + _Lang);		                                    
	} // end setupImplLanguage
	
	/**
	 * Sets up a connection to a Runtime Bridge class which provides access to the SCA runtime, enabling the runtime to be started
	 * and contributions to be passed to the runtime and started
	 * @return true if the Runtime Bridge was found and loaded, false if the bridge could not be found
	 * Uses a System Variable or an Environment Variable by the name "OASIS_TESTENV_RUNTIME_BRIDGE_CLASS" to name the Bridge class
	 */
	private boolean setupRuntimeBridge() throws Exception {

		String runtimeBridgeClass = configProperties.getProperty(RUNTIME_BRIDGE_IMPL_PROPNAME);
		if( runtimeBridgeClass == null ) {
			// Print message
			System.err.println("Set Environment Variable 'OASIS_TESTENV_RUNTIME_BRIDGE_CLASS' /n" +
					           "or set the property 'org.oasis.sca.tests.assembly.runtime_bridge' /n" +
					           "to the name of runtime bridge class "); 
		} // end if 
		
		if (runtimeBridgeClass != null) {
			try {
				runtimeBridge = (RuntimeBridge) Class.forName(
						runtimeBridgeClass).newInstance();
				return true;
			} catch (ClassNotFoundException e) {
				System.err.println("Runtime bridge class \"" + runtimeBridgeClass + "\" not found");
			}
		} // end if
		return false;
	} // end method setupRuntimeBridge
    
    public static void main(String[] args) throws Throwable {
    	BaseJAXWSTestRunner test = new BaseJAXWSTestRunner(); 
    	test.setUp();
    	test.tearDown();
    } // end method main
    
    /**
     * Set up the runtime - pass the configuration to the runtime bridge and then start the contribution(s) which represent
     * the SCA application
     * @throws Exception
     */
    @Before
    public void setUp() throws Throwable {
//    	runtimeBridge.setTestConfiguration(getTestConfiguration());
    	// Don't proceed to run the test unless the contribution loaded successfully... 
    	proceed = false;
    	try {
    		if ( startContribution() ) proceed = true; 
    	} catch (Exception e) {
    		// If the SCA runtime refuses to start an invalid contribution, then this is also
    		// regarded as a successful outcome, if the expected outcome is marked as "exception"
   			System.out.println( "Exception received during startup - detail: " + e.getMessage() );
            checkException(e);
    	} // end try
    }

    @After
    public void tearDown() throws Exception {
    	stopContribution();
    }

    /**
     * Run the test, if the initialization of the SCA application was successful
     * @throws Exception - if the output from the application does not match the expected output
     */
    @Test
    public void testDummy() throws Throwable {
    	// If an exception were thrown during initialization, let's go no further
    	if( proceed == false ) return; //TODO - check, should we be failing here?
    	
    	// System.out.println("Test " + testName + " starting");
    	String output = null;
    	try {
	    	output = invokeTest( testConfiguration.getInput() );

    	} catch ( TestException_Exception e ) {
    		TestException exceptionContent = e.getFaultInfo();
    		System.out.println("Service fault received - detail: " + exceptionContent.getMessage() );
            checkException(e);
    		return;
    	} catch (Throwable e) {
    		//e.printStackTrace();
   			System.out.println( "Exception received - detail: " + e.getMessage() );
            checkException(e);
    		return;
    	}
    	// If there are multiple outputs possible, check each one until there is a match
    	String[] expectedOutput = testConfiguration.getExpectedOutput();
    	AssertionError e = null;
    	
		for( String expected : expectedOutput ) {
			e = null;
    		try {
    			assertEquals( expected, output );
    		} catch ( AssertionError eReturned ) {
    			e = eReturned;
    		} //end try
    		if ( e == null ) break;
		} // end for 
		if( e != null ) throw e;
    	System.out.println("Test " + testConfiguration.getTestName() + " completed successfully");
    }
    
    /**
     * Invoke the test via a JAX-WS web services interface
     * @param input - input string
     * @return the output string from the web service invocation
     * @throws Exception - if the Web service invocation fails (e.g. the target service does not exist)
     */
    public String invokeTest( String input ) throws Exception {
    	//Web service invocation via JAXWS
    	QName serviceName = new QName("http://test.sca.oasisopen.org/", testConfiguration.getServiceName());
    	URL wsdlLocation = this.getClass().getClassLoader().getResource(testConfiguration.getWsdlFileName());
    	javax.xml.ws.Service webService = Service.create( wsdlLocation, serviceName );
    	TestInvocation wsProxy = (TestInvocation) webService.getPort(testConfiguration.getServiceInterface());

    	String output = wsProxy.invokeTest(input);

    	return output;
    } // end method invokeTest
    
    public void checkException(Throwable e) throws Throwable{
    	e.printStackTrace();
		assertEquals( testConfiguration.getExpectedOutput()[0], "exception" );
		
		runtimeBridge.checkError(testConfiguration.getTestName(), e);
		
		System.out.println("Test " + testConfiguration.getTestName() + " completed successfully");
        
    } // end method checkException

    /**
     * Load the supplied contribution(s) and load the named composite via the Runtime Bridge
     * @return true if the contributions were started successfully
     * @throws Exception
     */
    protected boolean startContribution() throws Exception {
    	
    	return runtimeBridge.startContribution(testConfiguration.getComposite(),
    			                               configProperties.getProperty(CONTRIBUTION_LOCATION),
    			                               testConfiguration.getContributionNames());
    	
    } // end method startContribution
    
    /**
     * Stop the SCA application
     * @throws Exception if there was a problem stopping the original application
     */
    protected void stopContribution() throws Exception {
    		runtimeBridge.stopContribution();
    } // end method stopContribution
    
    /**
     * The test configuration - this method is subclassed by individual testcases to provide detail about the test to run
     * @return a Test Configuration data structure
     */
    protected TestConfiguration getTestConfiguration() {
    	TestConfiguration config = new TestConfiguration();
    	// The name of the test
    	config.testName 		= "ASM_0001";
    	// The input string for the test
    	config.input 			= "request";
    	// The expected output string from the test
    	config.output[0] 			= config.testName + " " + config.input + " invoked ok";
    	// The name of the composite to run for the SCA application under test
    	config.composite 		= "Test_ASM_0101.composite";
    	// The test service to invoke on the SCA application side
    	config.testServiceName 	= "TestClient";
    	// An array of contribution names
    	config.contributionNames = new String[] { "General" };
    	// The client service invocation class
    	config.serviceInterface = TestInvocation.class;
    	return config;
    }
    
} // end class BaseTest
